# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_value_counts_plus.ipynb.

# %% auto 0
__all__ = ['value_counts_plus']

# %% ../nbs/00_value_counts_plus.ipynb 3
import pandas as pd

# %% ../nbs/00_value_counts_plus.ipynb 4
def value_counts_plus(
    series,
    dropna=False,
    show_top=10,
    sort_others=False,
    style=True,
    size=10,
    thousands=',',
    decimal='.',
    name='data',
    background_gradient='cividis'):
    """
    Provide a few ways of showing counts of values of items in ``series``.

    Parameters
    ----------
    series : pandas Series or list
        A sequence of items to count.
    dropna : bool
        Whether or not to drop missing values.
    show_top : int
        How many of the top rows to display.
    sort_others : bool
        Whether or not to place "Others" in the bottom (default) or in its
        sorted order position.
    style : bool
        Whether or not to style values for easier reading. If set to ``True``
        the result would not be a DataFrame, and cannot be further manipulated.
        Set the value to ``False`` to get aDataFrame as the return value.
    size : int
        The size in points of the font of the table. This results in the whole
        table being resized.
    thousands : str
        The character to use to separate thousands if `style=True`. Defaults to
        `,` but you can change to `.` or space, or any oher character you want.
    decimal : str
        The character to use to display decimal number if `style=True`. Defaults to
        `.` but you can change to `,`or any oher character you want.

    name : str
        The name of the column that you want displayed in the final table. It
        appears in the caption and defaults to "data".
    background_gradient: str
        The name of the color map to be used as the gradient. Many color maps
        are available: cividis, viridis, copper, cool, magma, and more. You can
        reverse the color by appending _r to the end of the colormap name
        cividis_r for example. Enter a random string to get an error message
        with all available colormaps.

    Returns
    -------
    value_counts_df : pandas.io.formats.style.Styler
        A DataFrame showing counts based on the provided arguments
    """
    final_col_names = ['count', 'cum_count', 'perc', 'cum_perc']
    if name in final_col_names:
        raise ValueError(f"Please make sure you use a name other than {final_col_names}")
    val_counts = pd.Series(series).rename(name).value_counts(dropna=dropna).reset_index()
    val_counts.columns = [name, 'count']
    if len(val_counts) > show_top:
        others_df = pd.DataFrame({
            name: ['Others:'],
            'count': val_counts[show_top:]['count'].sum()
            }, index=[show_top])
        val_counts = pd.concat([
            val_counts[:show_top],
            others_df
        ])
        if sort_others:
            val_counts = val_counts.sort_values(by=['count'], ascending=False)

    count_df = (val_counts
                .assign(
                    cum_count=lambda df: df['count'].cumsum(),
                    perc=lambda df: df['count'].div(df['count'].sum()),
                    cum_perc=lambda df: df['perc'].cumsum())
                )
    if not style:
        return count_df
    count_df.index = range(1, len(count_df)+1)
    count_df.columns = [name, 'count', 'cum. count', '%', 'cum. %']
    return (count_df
            .style
            .format({'count': '{:>,}',
                     'cum. count': '{:>,}',
                     '%': '{:>.1%}',
                     'cum. %': '{:>.1%}'},
                    thousands=thousands,
                    decimal=decimal)
            .background_gradient(background_gradient)
            .highlight_null()
            .set_caption(f'<h2>Counts of <b>{name}</b></h2>')
            .set_table_attributes(f'style=font-size:{size}pt;'))




__version__ = "0.0.13"

from .value_counts_plus import value_counts_plus
from .status_codes import status_codes
from .url_structure import url_structure
from .racing_chart import racing_chart
from .flag import flag, country_code_flag
from .style_table import style_table


# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/adviz',
                'doc_host': 'https://eliasdabbas.github.io',
                'git_url': 'https://github.com/eliasdabbas/adviz',
                'lib_path': 'adviz'},
  'syms': { 'adviz.core': {'adviz.core.foo': ('core.html#foo', 'adviz/core.py')},
            'adviz.flag': {'adviz.flag.flag': ('flag.html#flag', 'adviz/flag.py')},
            'adviz.racing_chart': {'adviz.racing_chart.racing_chart': ('racing_chart.html#racing_chart', 'adviz/racing_chart.py')},
            'adviz.status_codes': {'adviz.status_codes.status_codes': ('status_codes.html#status_codes', 'adviz/status_codes.py')},
            'adviz.style_table': { 'adviz.style_table._category_to_color': ('style_table.html#_category_to_color', 'adviz/style_table.py'),
                                   'adviz.style_table.style_table': ('style_table.html#style_table', 'adviz/style_table.py')},
            'adviz.url_structure': {'adviz.url_structure.url_structure': ('url_structure.html#url_structure', 'adviz/url_structure.py')},
            'adviz.value_counts_plus': { 'adviz.value_counts_plus.value_counts_plus': ( 'value_counts_plus.html#value_counts_plus',
                                                                                        'adviz/value_counts_plus.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/05_style_table.ipynb.

# %% auto 0
__all__ = ['style_table']

# %% ../nbs/05_style_table.ipynb 3
import pandas as pd
import plotly.express as px
import plotly.graph_objects as go
from plotly.subplots import make_subplots

# %% ../nbs/05_style_table.ipynb 4
def _category_to_color(categories, colorscale='D3'):
    colorscale = eval(f'px.colors.qualitative.{colorscale}')
    cat_dict = dict(enumerate(set(categories)))
    cat_dict = {v: colorscale[k] for k, v in cat_dict.items()}
    return [cat_dict[cat] for cat in categories]


# %% ../nbs/05_style_table.ipynb 5
def style_table(
    df,
    column_types,
    column_widths=None,
    title=None,
    precision=1,
    width=None,
    height=None,
    theme='plotly_white',
    font_size=None,
    title_font_size=None):
    """Convert a DataFrame to multiple charts (one for each column).

    Parameters
    ----------
    df : pandas.DataFrame
      Any DataFrame, but typically a small one, to display as a report.
    column_types : list
      A list of types, one for each column. Possible values are "bar", "heatmap", "category", and "text".
    column_widths : list
      A list of fractions that should add up to 1. Each fraction corresponds to a column.
    title : str
      The title of the chart.
    precision : int
      How many decimals of precision to display in numeric columns. This affects all numeric columns.
    width : int
      The width of the entire chart in pixels.
    height : int
      The height of the entire chart in pixels.
    theme : str
      The theme used for styling the entire chart.
    font_size : int
      The size of font of text and number on the chart in points.
    title_font_size : int
      The size of font of the title of the chart in points.
    Returns
    -------
    styled_table : plotly.graph_objects.Figure
      A Plotly Figure object that can be retroactively edited if desired.
    """
    if not set(column_types).issubset(["bar", "heatmap", "category", "text"]):
        raise ValueError("Please make sure you specify any of the following types:\n"
                         "'bar', 'heatmap', 'category', and 'text'")
    fig = make_subplots(
        rows=1,
        cols=df.shape[1],
        column_widths=column_widths,
        horizontal_spacing=0,
        subplot_titles=df.columns)
    categorical_scales = [data.y[0] for data in px.colors.qualitative.swatches().data][::-1]
    categorical_index = 0
    texttemplate = f'%{{text:,.{precision}f}}'
    for i, col in enumerate(df):
        if column_types[i] == 'heatmap':
            fig.add_heatmap(
                z=df[[col]],
                name=col,
                showscale=False,
                colorscale='cividis',
                hovertemplate=texttemplate,
                text=df[[col]],
                texttemplate=texttemplate,
                row=1,
                col=i+1)
        elif column_types[i] == 'bar':
            fig.add_bar(
                x=df[col],
                y=list(range(len(df))),
                showlegend=False,
                orientation='h',
                col=i+1,
                marker={'color': 'steelblue'},
                text=df[col],
                texttemplate=texttemplate,
                hovertemplate=texttemplate,
                row=1,
                name=col)
        elif column_types[i] == 'category':
            categorical_index += 1
            fig.add_bar(
                x=[10 for i in range(len(df))],
                y=list(range(len(df))),
                showlegend=False,
                marker={'color': _category_to_color(
                    df[col],
                    categorical_scales[categorical_index] if df[col].nunique() <= 10 else 'Dark24'),
                        'opacity': 1},
                row=1,
                col=i+1,
                name=col,
                orientation='h',
                text=df[col],
                hovertemplate='<b>%{text}')
        elif column_types[i] == 'text':
            fig.add_bar(
                x=[10 for i in range(len(df))],
                y=list(range(len(df))),
                orientation='h',
                showlegend=False,
                name=col,
                hovertemplate='<b>%{text}',
                marker={'color': 'white'},
                text=df[col],
                texttemplate='%{text}',
                row=1,
                col=i+1)
    fig.update_xaxes(showticklabels=False, showgrid=False, zeroline=False)
    fig.update_yaxes(showticklabels=False, showgrid=False, zeroline=False, autorange='reversed')
    fig.update_layout(bargap=0, template=theme, width=width, height=height,
                      title=title, font_size=font_size, title_font_size=title_font_size)
    return fig



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/02_url_structure.ipynb.

# %% auto 0
__all__ = ['url_structure']

# %% ../nbs/02_url_structure.ipynb 3
import advertools as adv
import adviz
import pandas as pd
pd.options.display.max_colwidth = 120
import plotly.express as px

# %% ../nbs/02_url_structure.ipynb 4
_texttemplate = '<b>%{label} </b><br><br>Directory: <b>%{parent}/%{label}</b><br>Count: %{value:,}<br>%{percentParent:.1%} of %{parent}<br>'

# %% ../nbs/02_url_structure.ipynb 5
def url_structure(
    url_list,
    items_per_level=10,
    height=600,
    width=None,
    theme='none',
    domain='example.com',
    title='URL Structure'):
    """
    Create a treemap for the first two URL path directories `example.com/dir_1/dir_2/`.

    Parameters
    ----------
    url_list : list
        Any list-like object with a bunch of URLs.
    items_per_level : int
        The number of items to display for each level of the treemap. All other
        items will be grouped under a special item called "Others".
    height : int
        The height of the chart in pixels.
    width : int
        The width of the chart in pixels.
    theme : str
        Name of theme to use for the chart. Available themes:
            ggplot2, seaborn, simple_white, plotly, plotly_white, plotly_dark,
            presentation, xgridoff, ygridoff, gridon, none.
    domain : str
        The main domain of the URL list. This will be displayed at the top
        panel in the treemap to display values like a breadcrumb.
    title: str
        The title of the figure. You can use/include the following HTML tags in
        the title: `<a>`, `<b>`, `<br>`, `<i>`, `<sub>`, `<sup>`

    Returns
    -------
    url_structure_treemap : plotly.graph_objects.Figure
    """
    urldf = adv.url_to_df(url_list)
    dir1_top_n = urldf['dir_1'].value_counts().head(items_per_level).index.tolist() + ['Others:']
    urldf['dir_1_clean'] = [x if x in dir1_top_n else 'Others:' for x in urldf['dir_1']]
    top_n_df = pd.DataFrame(dir1_top_n, columns=['dir_1_top_n'])

    dir2_valcounts = []

    for top_n in top_n_df['dir_1_top_n']:
        tempdf = urldf[urldf['dir_1_clean'].eq(top_n)]
        valcountsdf = adviz.value_counts_plus(tempdf['dir_2'], show_top=items_per_level, style=False)
        valcountsdf = valcountsdf.assign(dir_1_value=top_n)
        dir2_valcounts.append(valcountsdf)
    dir2_df = pd.concat(dir2_valcounts, ignore_index=True)[['data', 'count', 'dir_1_value']]
    treemap_df = pd.merge(
        top_n_df,
        dir2_df,
        left_on='dir_1_top_n',
        right_on='dir_1_value',
        how='left')
    fig = px.treemap(
        treemap_df.dropna(), 
        path=[px.Constant(domain), 'dir_1_top_n', 'data'], 
        branchvalues='total',
        maxdepth=2,
        width=width,
        height=height,
        title=title,
        template=theme,
        values='count')
    fig.data[0].marker.line.width = 0.01
    fig.data[0].marker.pad = dict.fromkeys('lrbt', 0)
    fig.data[0]['texttemplate'] = _texttemplate
    fig.data[0]['hovertemplate'] = _texttemplate
    fig.update_traces(pathbar={'edgeshape': '/'})
    return fig



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/03_racing_chart.ipynb.

# %% auto 0
__all__ = ['racing_chart']

# %% ../nbs/03_racing_chart.ipynb 3
import pandas as pd
import plotly.express as px
from plotly.subplots import make_subplots
import adviz

# %% ../nbs/03_racing_chart.ipynb 4
def racing_chart(
    df,
    n=10,
    title="Racing Chart",
    frame_duration=500,
    theme='none',
    width=None,
    height=600,
    font_size=12):
    """
    Create a racing bar chart showing the top `n` values for each period.

    Parameters
    ----------
    df : pandas.DataFrame
      A DataFrame containing three columns for entity, metric, and period. The
      names can be anything bu they have to be in this specific order.
    n : int
      The number of top items to display for each period.
    title : str
      The title of the chart.
    frame_duration : int
      The duration of each frame during animation, before transitioning to the
      following frame, in milliseconds.
    theme : str
      The theme of the chart. Any of the following values can be used:
    width : int
      The width of the chart in pixels.
    height : int
      The height of the chart in pixels.
    font_size : int
      The size of the fonts in the chart.

    Returns
    -------
    racing_barchart : plotly.graph_objects.Figure
    """
    period_totals = df.groupby(df.columns[2]).sum(df.columns[1]).reset_index()
    # df[f'{df.columns[1]} %'] = df.groupby(df.columns[2])[df.columns[1]].transform(lambda x: x / x.sum())
    top_n_df = (df
        .sort_values([df.columns[2], df.columns[1]],
                     ascending=[True, False])
        .groupby(df.columns[2])
        .head(n))
    fig = px.bar(
        top_n_df,
        x=df.columns[1],
        y=df.columns[0],
        text=df.columns[1],
        animation_frame=df.columns[2],
        hover_name=df.columns[0],
        height=height,
        width=width,
        title=title,
        template=theme,
        range_x=[0,top_n_df[df.columns[1]].max() * 1.05],
    )
    fig.layout.yaxis.autorange = 'reversed'
    fig.layout.font.size = font_size
    fig.layout.sliders[0].currentvalue.xanchor = 'right'
    fig.layout.sliders[0].currentvalue.prefix = fig.layout.sliders[0].currentvalue.prefix.replace('=', ': ')
    fig.layout.sliders[0].currentvalue.font.size = font_size * 1.5
    fig.layout.sliders[0].bgcolor = '#D5D5D5'
    fig.layout.sliders[0].borderwidth = 0
    fig.layout.updatemenus[0]['buttons'][0]['args'][1]['frame']['duration'] = frame_duration
    for step, total in zip(fig.layout.sliders[0].steps, period_totals.iloc[:, 1]):
        step['label'] = f"{step['label']}<br>Total: {total:,}"
    return fig



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_status_codes.ipynb.

# %% auto 0
__all__ = ['status_codes']

# %% ../nbs/01_status_codes.ipynb 4
from http.client import responses

import pandas as pd
import plotly.express as px

# %% ../nbs/01_status_codes.ipynb 5
_texttemplate = '<b>%{label} </b><br><br>Status code: %{label} %{customdata[0]}<br>Count: %{value:,}<br>%{percentParent:.1%} of %{parent}<br>' #%{percentRoot:.1%} of %{root}'

# %% ../nbs/01_status_codes.ipynb 6
def status_codes(
    status_list,
    height=600,
    width=None,
    theme='none',
    title='Status Codes',
    export_to_html=None):
    """Create a treemap to visualize a list of status codes.

    Parameters
    ----------

    status_list : list, tuple, pandas.Series
        A collection of HTTP status codes
    height: integer
        The desired height of the figure in pixels
    width: integer
        The desired width of the figure in pixels
    theme : str
        Name of theme to use for the chart. Available themes:
            ggplot2, seaborn, simple_white, plotly, plotly_white, plotly_dark,
            presentation, xgridoff, ygridoff, gridon, none
    title: str
        The title of the figure. You can use/include the following HTML tags in
        the title: `<a>`, `<b>`, `<br>`, `<i>`, `<sub>`, `<sup>`
    export_to_html: str, optional
        The path to the HTML file if you want to save it as a shareable file

    Returns
    -------
    status_codes_figure : plotly.graph_objects.Figure
    """
    status_counts = pd.Series(status_list).value_counts().reset_index()
    status_counts.columns = ['status', 'count']
    status_counts = (status_counts
        .assign(status_cat=lambda df: df['status'].astype(int).round(-2))
        .assign(status_desc=lambda df: [responses[int(code)] for code in df['status']])
    )
    status_fig = px.treemap(
        status_counts,
        maxdepth = 2,
        path=[px.Constant('Status codes'), 'status_cat', 'status'],
        hover_name='status_cat',
        height=height,
        width=width,
        custom_data=['status_desc'],
        values='count',
        title=title,
        template=theme
    )
    status_fig.data[0].marker.line.width = 0.01
    status_fig.data[0].marker.pad = dict.fromkeys('lrbt', 0)
    status_fig.data[0]['texttemplate'] = _texttemplate
    status_fig.data[0]['hovertemplate'] = _texttemplate
    if export_to_html:
        status_fig.write_html(export_to_html)
    return status_fig
